// jQuery WipeTouch 1.2.0
//监测屏幕手势

(function($) {
	$.fn.wipetouch = function(settings) {

		var config = {

			moveX: 40,
			moveY: 40,
			tapToClick: false,
			preventDefault: true,
			allowDiagonal: false,

			wipeLeft: false,
			wipeRight: false,
			wipeUp: false,
			wipeDown: false,
			wipeUpLeft: false,
			wipeDownLeft: false,
			wipeUpRight: false,
			wipeDownRight: false,
			wipeMove: false,

			wipeTopLeft: false,
			wipeBottomLeft: false,
			wipeTopRight: false,
			wipeBottomRight: false
		};

		if (settings) {
			$.extend(config, settings);
		}

		this.each(function() {
			var startX;
			var startY;
			var startDate = false;
			var curX;
			var curY;
			var isMoving = false;
			var touchedElement = false;

			var useMouseEvents = false;
			var clickEvent = false;

			// Called when user touches the screen.
			function onTouchStart(e) {
				var start = useMouseEvents || (e.originalEvent.touches && e.originalEvent.touches.length > 0);

				if (!isMoving && start) {
					if (config.preventDefault) {
						e.preventDefault();
					}

					if (config.allowDiagonal) {
						if (!config.wipeDownLeft) {
							config.wipeDownLeft = config.wipeBottomLeft;
						}

						if (!config.wipeDownRight) {
							config.wipeDownRight = config.wipeBottomRight;
						}

						if (!config.wipeUpLeft) {
							config.wipeUpLeft = config.wipeTopLeft;
						}

						if (!config.wipeUpRight) {
							config.wipeUpRight = config.wipeTopRight;
						}
					}

					// When touch events are not present, use mouse
					// events.
					if (useMouseEvents) {
						startX = e.pageX;
						startY = e.pageY;

						$(this).bind("mousemove", onTouchMove);
						$(this).one("mouseup", onTouchEnd);
					} else {
						startX = e.originalEvent.touches[0].pageX;
						startY = e.originalEvent.touches[0].pageY;

						$(this).bind("touchmove", onTouchMove);
					}

					// Set the start date and current X/Y.
					startDate = new Date().getTime();
					curX = startX;
					curY = startY;
					isMoving = true;

					touchedElement = $(e.target);
				}
			}

			// Called when user untouches the screen.
			function onTouchEnd(e) {
				if (config.preventDefault) {
					e.preventDefault();
				}

				// When touch events are not present, use mouse events.
				if (useMouseEvents) {
					$(this).unbind("mousemove", onTouchMove);
				} else {
					$(this).unbind("touchmove", onTouchMove);
				}

				// If is moving then calculate the touch results,
				// otherwise reset it.
				if (isMoving) {
					touchCalculate(e);
				} else {
					resetTouch();
				}
			}

			// Called when user is touching and moving on the screen.
			function onTouchMove(e) {
				if (config.preventDefault) {
					e.preventDefault();
				}

				if (useMouseEvents && !isMoving) {
					onTouchStart(e);
				}

				if (isMoving) {
					if (useMouseEvents) {
						curX = e.pageX;
						curY = e.pageY;
					} else {
						curX = e.originalEvent.touches[0].pageX;
						curY = e.originalEvent.touches[0].pageY;
					}

					// If there's a wipeMove event, call it passing
					// current X and Y position (curX and curY).
					if (config.wipeMove) {
						triggerEvent(config.wipeMove, {
							curX: curX,
							curY: curY
						});
					}
				}
			}

			function touchCalculate(e) {
				var endDate = new Date().getTime(); // current date to
				// calculate timing
				var ms = startDate - endDate; // duration of touch in
				// milliseconds

				var x = curX; // current left position
				var y = curY; // current top position
				var dx = x - startX; // diff of current left to
				// starting left
				var dy = y - startY; // diff of current top to
				// starting top
				var ax = Math.abs(dx); // amount of horizontal movement
				var ay = Math.abs(dy); // amount of vertical movement

				if (ax < 15 && ay < 15 && ms < 100) {
					clickEvent = false;

					if (config.preventDefault) {
						resetTouch();

						touchedElement.trigger("click");
						return;
					}
				}
				// When touch events are not present, use mouse events.
				else if (useMouseEvents) {
					var evts = touchedElement.data("events");

					if (evts) {
						// Save click event to the temp clickEvent
						// variable.
						var clicks = evts.click;

						if (clicks && clicks.length > 0) {
							$.each(clicks, function(i, f) {
								clickEvent = f;
								return;
							});

							touchedElement.unbind("click");
						}
					}
				}

				// Is it moving to the right or left, top or bottom?
				var toright = dx > 0;
				var tobottom = dy > 0;

				// Calculate speed from 1 to 5, 1 being slower and 5
				// faster.
				var s = ((ax + ay) * 60) / ((ms) / 6 * (ms));

				if (s < 1)
					s = 1;
				if (s > 5)
					s = 5;

				var result = {
					speed: parseInt(s),
					x: ax,
					y: ay,
					source: touchedElement
				};

				if (ax >= config.moveX) {
					// Check if it's allowed and trigger diagonal wipe
					// events.
					if (config.allowDiagonal && ay >= config.moveY) {
						if (toright && tobottom) {
							triggerEvent(config.wipeDownRight, result);
						} else if (toright && !tobottom) {
							triggerEvent(config.wipeUpRight, result);
						} else if (!toright && tobottom) {
							triggerEvent(config.wipeDownLeft, result);
						} else {
							triggerEvent(config.wipeUpLeft, result);
						}
					}
					// Otherwise trigger horizontal events if X > Y.
					else if (ax >= ay) {
						if (toright) {
							triggerEvent(config.wipeRight, result);
						} else {
							triggerEvent(config.wipeLeft, result);
						}
					}
				}
				// If Y > X and no diagonal, trigger vertical events.
				else if (ay >= config.moveY && ay > ax) {
					if (tobottom) {
						triggerEvent(config.wipeDown, result);
					} else {
						triggerEvent(config.wipeUp, result);
					}
				}

				resetTouch();
			}

			// Resets the cached variables.
			function resetTouch() {
				startX = false;
				startY = false;
				startDate = false;
				isMoving = false;

				// If there's a click event, bind after a few
				// miliseconds.
				if (clickEvent) {
					window.setTimeout(function() {
						touchedElement.bind("click", clickEvent);
						clickEvent = false;
					}, 50);
				}
			}

			function triggerEvent(wipeEvent, result) {
				if (wipeEvent) {
					wipeEvent(result);
				}
			}

			if ("ontouchstart" in document.documentElement) {
				$(this).bind("touchstart", onTouchStart);
				$(this).bind("touchend", onTouchEnd);
			} else {
				useMouseEvents = true;

				$(this).bind("mousedown", onTouchStart);
				$(this).bind("mouseout", onTouchEnd);
			}
		});

		return this;
	};
})(jQuery);